// $Id$

//=============================================================================
/**
 *  @file    enumerator.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_ENUMERATOR_H
#define JYUGEM_BEE_ENUMERATOR_H

#include "jyugem/bee.h"
#include <boost/tokenizer.hpp>

namespace jyugem { namespace bee
{

//
// Enumerate Type
//
class BEE_Export enumerator : public nameable
{
public:
    // Enumerate element
    class enum_element
    {
        friend class enumerator;
    public:
        enum_element(const std::string& name, int num)
            : m_name(name), m_num(num) {}
        enum_element(const enum_element& rhs)
        {
            m_name = rhs.m_name;
            m_num = rhs.m_num;
        }
        enum_element& operator=(const enum_element& rhs)
        {
            if (this == &rhs)
                return *this;
            m_name = rhs.m_name;
            m_num = rhs.m_num;
            return *this;
        }

        int value() const         { return m_num; }
        std::string& name() const { return m_name; }

    protected:
        enum_element() : m_name(_TX("")), m_num(0) {}

    protected:
        std::string m_name;
        int         m_num;
    };

    //
    typedef sequential_map<enum_element, std::string, &enum_element::name >
        enumerator_map;

    // Constructor
    enumerator(const std::string& name) : nameable(name) {}
    enumerator(const std::string& name, strcode initv[])
    {
        int i = 0;
        while (initv[i].m_str != null)
        {
            this->add(initv[i].m_str, initv[i].m_num);
            i++;
        }
    }
    enumerator(const std::string& name, vector<std::string> initv)
    {
        for (size_t i = 0; i < initv.size(); i++)
        {
            this->add(initv[i], i);
        }
    }
    enumerator(const std::string& name, const std::string& initv)
    {
        typedef boost::tokenizer<boost::char_separator<char> > Tok;
        boost::char_separator<char> sep(","); // default constructed
        Tok tok(key, sep);  // default separater are space or punctuation
        try
        {
            int i = 0;
            for (Tok::iterator beg = tok.begin(); beg != tok.end(); ++beg)
            {
                this->add(*beg, i);
                i++;
            }
            catch (boost::bad_lexical_cast& ex)
            {
                TRACE_ERROR((_TX("%s\n"), ex.what()));
                throw ;
            }
        }
    }
    enumerator(const enumerator& rhs) : nameable(rhs)
    {
        m_elms = rhs.m_elms;
    }

    // Destructor
    virtual ~enumerator() {}

    // Get enum number by name
    bool find(const std::string& ename, int& retnum)
    {
        enum_element * elm = m_elms.find(ename);
        if (elm == null)
        {
            return false;
        }
        retnum = elm->value();
        return true;
    }

    // Get enum name by number
    bool find(int num, std::string& ename)
    {
        enum_element * elm = m_elms.get(num);
        if (elm == null || (elm != null && elm->value() != num))
        {
            for (int i = 0; i < m_elms.size(); i++)
            {
                elm = m_elms.get(i);
                if (elm != null && elm->value() == num)
                {
                    ename = elm->name();
                    return true;
                }
            }
            return false;
        }
        if (elm == null)
        {
            return false;
        }
        ename = elm->name();
        return true;
    }


//
protected:
    enumerator() : nameable() {}
    // Add enumerator element
    int add(const std::string& mbr, int num)
    {
        m_elms.add(new enum_element(mbr, num));
    }

protected:
    enumerator_map m_elms;
};

} } // *namespace* jyugem::bee

#endif  // JYUGEM_BEE_ENUMERATOR_H
