// $Id$

//=============================================================================
/**
 *  @file  id.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_ID_H
#define JYUGEM_BEE_ID_H

#include "jyugem/bee.h"


namespace jyugem { namespace bee
{

//
// Identifier class
//
class BEE_Export id : public bee::value
{
public:
  // Constructor
  id() {}
  id(int i)   : bee::value(i) {}
  id(UINT i)  : bee::value(i) {}
  id(const std::string& i) : bee::value(i) {}

  id(const id& rhs) : value(rhs) {}
  ~id() {}

  id& operator=(int i)  { this->bee::value::operator=(i); return *this; }
  id& operator=(UINT val) { this->bee::value::operator=(val); return *this; }
  id& operator=(const std::string& val)
  { this->bee::value::operator=(val); return *this; }

  id& operator=(const id& rhs)
  {
    if (this == &rhs)
      return *this;
    this->bee::value::operator=(rhs);
    return *this;
  }

  friend bool operator==(const id& lhs, const id& rhs);
  friend bool operator!=(const id& lhs, const id& rhs)
  { return (! (lhs == rhs)); }

  friend bool operator<(const id& lhs, const id& rhs);

//
protected:

};

// Operation
inline bool operator==(const id& lhs, const id& rhs)
{
  return (static_cast<const value&>(lhs) == static_cast<const value&>(rhs));
}

inline bool operator<(const id& lhs, const id& rhs)
{
  return (static_cast<const value&>(lhs) < static_cast<const value&>(rhs));
}


// Compare operation
struct id_less
{
  bool operator() (const id& x, const id& y) const
  {
    return (x < y);
  }
};

//
// cintainer of values
//
class Ids : std::vector<id>
{
public:
  Ids() {}
  ~Ids() {}

  id get(int i)
  {
    if ((size_t)i >= size())
      return id();
    return (*this)[i];
  }
  int add(const id& v)
  {
    this->push_back(v);
    return 0;
  }
  int set(int i, const id& v)
  {
    if ((size_t)i >= size())
      return -1;
    (*this)[i] = v;
    return 0;
  }
};

} } // *namespace*

#endif  /* JYUGEM_BEE_ID_H */
