// $Id$

//=============================================================================
/**
 *  @file    mapped_nameable.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_MAPPED_NAMEABLE_H
#define JYUGEM_BEE_MAPPED_NAMEABLE_H

#include "jyugem/bee.h"

namespace jyugem { namespace bee
{

//
// Interface class
//
template<class T>
class BEE_Export mapped_nameable : public nameable
{
public:
    mapped_nameable(const std::string& name) : nameable(name)
    {
        T::insert(name, this);
    }
    mapped_nameable(const nameable& rhs) : nameable(rhs)
    {
        T::insert(rhs.m_uniqueName, this);
    }
    ~mapped_nameable()
    {
        T::remove(this->m_uniqueName, this);
    }

    mapped_nameable& operator=(const mapped_nameable& rhs)
    {
        if (this == &rhs)
            return *this;
        m_uniqueName = rhs.m_uniqueName;
        return *this;
    }

    friend bool operator==(const mapped_nameable& lhs, const mapped_nameable& rhs);
    friend bool operator<(const mapped_nameable& lhs, const mapped_nameable& rhs);

protected:
    mapped_nameable() : nameable(_TX("")) {}

};

//
// Operational functions
//
inline bool operator<(const mapped_nameable& lhs, const mapped_nameable& rhs)
{
    return (lhs.m_uniqueName < rhs.m_uniqueName);
}
inline bool operator==(const mapped_nameable& lhs, const mapped_nameable& rhs)
{
    return (lhs.m_uniqueName == rhs.m_uniqueName);
}


} } // *namespace*

#endif  /* JYUGEM_BEE_MAPPED_NAMEABLE_H */
