// $Id$

//=============================================================================
/**
 *  @file  nameable.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_NAMEABLE_H
#define JYUGEM_BEE_NAMEABLE_H

#include "jyugem/bee.h"
#include "jyugem/bee/uuid.h"

namespace jyugem { namespace bee
{

//
// Interface class
//
class BEE_Export nameable
{
public:
  typedef map<std::string, nameable *, less<std::string> > dictionary;

  nameable(const std::string& name) : m_uniqueName(name) {}
  nameable(const nameable& rhs) : m_uniqueName(rhs.m_uniqueName) {}
  ~nameable() {}

  nameable& operator=(const nameable& rhs)
  {
    if (this == &rhs)
      return *this;
    m_uniqueName = rhs.m_uniqueName;
    return *this;
  }

  const std::string& uniqueName() const { return m_uniqueName; }
  const std::string& uniqueName() { return m_uniqueName; }
  const BCHAR * name_p() const    { return m_uniqueName.c_str(); }

  template<class T>
  static T * downcast(const nameable * nmptr)
  {
    T * result = null;
    try
    {
      result = dynamic_cast<T *>(const_cast<nameable *>(nmptr));
    }
    catch (const std::bad_cast& ex)
    {
      TRACE_FAIL((_TX("Failure cast(%s): %s\n"), nmptr->name_p(), ex.what()));
    }
    return result;
  }


  friend bool operator==(const nameable& lhs, const nameable& rhs);
  friend bool operator<(const nameable& lhs, const nameable& rhs);

protected:
  nameable() : m_uniqueName(_TX("")) {}
  void  uniqueName(const std::string& nm) { m_uniqueName = nm; }

protected:
  std::string m_uniqueName;
  uuid        m_uuid;
};

//
// Operational functions
//
inline bool operator<(const nameable& lhs, const nameable& rhs)
{
  return (lhs.m_uniqueName < rhs.m_uniqueName);
}
inline bool operator==(const nameable& lhs, const nameable& rhs)
{
  return (lhs.m_uniqueName == rhs.m_uniqueName);
}

//
// Nameable map
//
class nameable_map
{
public:
  template<class T>
  T * find(const std::string& name)
  {
    nameable::dictionary::iterator iter = m_map.find(name);
    if (iter == m_map.end())
    {
      return null;
    }

    T * result = nameable::downcast<T>(iter->second);
    return result;
  }
  template<>
  nameable * find(const std::string& name)
  {
    nameable::dictionary::iterator iter = m_map.find(name);
    if (iter == m_map.end())
    {
      return null;
    }
    return iter->second;
  }

  int add(nameable * nameptr)
  {
    m_map.insert(std::make_pair(nameptr->uniqueName(), nameptr));
    return 0;
  }

  nameable * remove(const std::string& name)
  {
    nameable::dictionary::iterator iter = m_map.find(name);
    if (iter == m_map.end())
    {
      return null;
    }
    nameable * result = iter->second;
    m_map.erase(iter);
    return result;
  }

  nameable::dictionary m_map;
};

} } // *namespace* - jyugem::gem

#endif
