// $Id$

//=============================================================================
/**
 *  @file  sequential_map.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================


#ifndef JYUGEM_BEE_SEQUENTIAL_MAP_H
#define JYUGEM_BEE_SEQUENTIAL_MAP_H

#include "nameable.h"

namespace jyugem { namespace bee
{
//
// vector and map table
//
template<typename T, typename K, const string& (T::* getkey_func)() >
struct sequential_map : vector<T *>
{
  typedef K      key_type;
  typedef vector<T *>  base_type;

  typedef map<key_type, int, std::less<key_type> > keymap;

  sequential_map() {}
  sequential_map(const sequential_map& rhs)
    : base_type(rhs), m_map(rhs.m_map) {}
  ~sequential_map() {}

  sequential_map& operator=(const sequential_map& rhs)
  {
    if (this == &rhs)
      return *this;
    std::copy(rhs.begin(), rhs.end(), this->begin());
    m_map = rhs.m_map;
    return *this;
  }

  value_type find(const key_type& key)
  {
    keymap::iterator iter = m_map.find(key);
    if (iter == m_map.end())
      return null;
    int idx = (*iter).second;
    BEE_ASSERT(idx < (int)this->size());
    return (*this)[idx];
  }

  value_type get(int idx)
  {
    return ((size_t)idx >= this->size()) ? null : m_vector[idx];
  }

  int add(value_type elm)
  {
    this->push_back(elm);
    m_map.insert(std::make_pair((elm->*getkey_func)(), int(this->size()) - 1));
    return 0;
  }

  value_type remove(const key_type& key)
  {
    int idx = 0;
    map_type::iterator mapiter = m_map.find(key);
    if (mapiter == m_map.end())
      return null;
    idx = (*mapiter).second;
    BEE_ASSERT(idx < this->size());

    value_type result = (*this)[idx];
    this->erase(this->begin() + idx);
    m_map.erase(mapiter);

    mapiter = m_map.begin();
    for ( ; mapiter != m_map.end(); ++mapiter)
    {   // re-index
      if ((*mapiter).second > idx)
        (*mapiter).second--;
    }
    return result;
  }

  value_type remove(int idx)
  {
    if ((size_t) idx >= m_vector.size())
      return null;
    value_type result = (*this)[idx];
    this->remove((result->*getkey_func)());
    return result;
  }

protected:
  keymap   m_map;
};

//
// Nameable sequential map
//
typedef sequential_map<nameable, std::string, &nameable::uniqueName >
  attributes_map;

} } // *namespace*

#endif  /* JYUGEM_BEE_SEQUENTIAL_MAP_H */
