// $Id$

//=============================================================================
/**
 *  @file    ControlJob.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================
#ifndef JYUGEM_CJM_CONTROLJOB_H
#define JYUGEM_CJM_CONTROLJOB_H

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif


#include "jyugem/bee/state_task.h"
#include <ctime>
#include <iostream>
#include <string>

namespace bee = jyugem::bee;


//
namespace jyugem { namespace cjm
{

typedef bee::parameter Parameter;
//typedef CEM::SubstLocation SubstLoc;
typedef std::string SubstLoc;

//
// E94 : PROVISIONAL SPECIFICATION FOR CONTROL JOB MANAGEMENT
//

class ControlRule : public bee::parameter
{
public:
  ControlRule() : Parameter() {}
  ControlRule(const std::string& name, const bee::value& v)
    : Parameter(name, v) {}
  ControlRule(const ControlRule& rhs)
    : Parameter(static_cast<const Parameter&>(rhs)) {}
  ~ControlRule() {}

  ControlRule& operator=(const ControlRule& rhs)
  {
    if (this == &rhs)
       return *this;
    this->bee::parameter::operator=(static_cast<const bee::parameter&>(rhs));
    return *this;
  }

  std::string RuleName()  { return m_uniqueName; }
  bee::value  RuleValue() { return m_value; }

};

//
//
//
class CarrierMap
{
public:
  CarrierMap() : m_CarrierID(_TX("")), m_location(null) {}
  CarrierMap(bee::id& ind, SubstLoc * loc)
    : m_CarrierID(ind), m_location(loc) {}
  CarrierMap(const CarrierMap& rhs)
    : m_CarrierID(rhs.m_CarrierID), m_location(rhs.m_location) {}
  ~CarrierMap() {}

  CarrierMap& operator=(const CarrierMap& rhs)
  {
    if (this == &rhs)
      return *this;
    m_CarrierID = rhs.m_CarrierID;
    m_location = rhs.m_location;
    return *this;
  }

  std::string CarrierID() { return m_CarrierID.toString(); }
  int         LocID()     { return 0 /* m_location->locid() */; }

  bee::id m_CarrierID;
  SubstLoc * m_location;
};

//
//
//
class MtrlIOMap
{
public:
  MtrlIOMap() {}
  MtrlIOMap(const CarrierMap& inp, const CarrierMap& outp)
    : m_in(inp), m_out(outp) {}
  MtrlIOMap(const MtrlIOMap& rhs) : m_in(rhs.m_in), m_out(rhs.m_out) {}
  ~MtrlIOMap() {}

  MtrlIOMap& operator=(const MtrlIOMap& rhs)
  {
    if (this == &rhs)
      return *this;
    m_in = rhs.m_in;
    m_out = rhs.m_out;
    return *this;
  }

  CarrierMap m_in;
  CarrierMap m_out;
};

//
//
//
class MtrlOutRule
{
public:
  MtrlOutRule() : m_dest(_TX("")), m_MaterialStatus(0) {}
  MtrlOutRule(const std::string& dest, int status = 0)
    : m_dest(dest), m_MaterialStatus(status) {}
  MtrlOutRule(const MtrlOutRule& rhs)
    : m_dest(rhs.m_dest), m_MaterialStatus(rhs.m_MaterialStatus) {}
  ~MtrlOutRule() {}

  MtrlOutRule& operator=(const MtrlOutRule& rhs)
  {
    if (this == &rhs)
      return *this;
    m_dest = rhs.m_dest;
    m_MaterialStatus = rhs.m_MaterialStatus;
    return *this;
  }

  std::string m_dest;
  int m_MaterialStatus;
};

//
//
//
class ProcessRule
{
public:
  ProcessRule() : m_PRJobID(0), m_OutputRule(_TX("")) {}
  ProcessRule(const bee::id& jobid, const std::string& rule)
    : m_PRJobID(jobid), m_OutputRule(rule) {}
  ProcessRule(const ProcessRule& rhs)
    : m_PRJobID(rhs.m_PRJobID), m_OutputRule(rhs.m_OutputRule),
      m_ControlRule(rhs.m_ControlRule) {}
  ~ProcessRule() {}

  ProcessRule& operator=(const ProcessRule& rhs)
  {
    if (this == &rhs)
      return *this;
    m_PRJobID = rhs.m_PRJobID;
    m_OutputRule = rhs.m_OutputRule;
    m_ControlRule = rhs.m_ControlRule;
    return *this;
  }

  bee::id     m_PRJobID;
  std::string m_OutputRule;
  ControlRule m_ControlRule;
};

//
//
//
class ControlJob : public bee::state_task
{
  friend class ControlJobState;

public:
  ControlJob();
  // ControlJob(const ControlJob& rhs) : bee::state_task(rhs.name()) { this->copy(rhs); }
  virtual ~ControlJob();

  // ControlJob& operator=(const ControlJob& rhs)
  // {
  //   if ( this== &rhs)
  //     return *this;
  //   this->copy(rhs);
  //   return *this;
  // }

  enum { LIST, ARRIVAL, OPTIMIZE };   // ProcessOrderMgmt

  bool autoStart() { return m_StartMethod; }

  int open0();

  const BCHAR * current_state();

protected:
  // void copy(const ControlJob& rhs);

public:
  // std::vector<pjm::ProcessJob *> m_CurrentPRJob;
  std::vector<std::string> m_CurrentPRJob;
  bee::Ids                 m_CarrierInputSpec;
  bee::Ids                 m_PauseEvent;
  std::vector<MtrlIOMap *>   m_MtrlOutSpec;
  std::vector<MtrlOutRule *> m_MtrlOutbyStatus;
  std::vector<ProcessRule *> m_ProcessingCtrlSpec;
  std::string m_DataCollectionPlan;
  int  m_ProcessOrderMgmt;
  bool m_StartMethod;       // true = Auto, false = UserStart
  int  m_State;
};


} }  // *namespace* jyugem::cjm

#endif  /* JYUGEM_CJM_CONTROLJOB_H */

