// $Id$

//=============================================================================
/**
 *  @file    datetime.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "jyugem/bee/datetime.h"
using namespace jyugem::bee;

#include "time.h"

namespace jyugem { namespace bee
{

//-----------------------------------------------------------------------------
// Get time value of ACE format.
//-----------------------------------------------------------------------------
ACE_Time_Value datetime::value()
{
  // struct tm tv;
  // long   millisec;
  // datetime::to_localtime(m_tv, tv, millisec);
  // return ACE_Time_Value(mktime(&tv), millisec * 1000);

  ptime timet_start(date(1970,1,1));
  time_duration diff = m_tv - timet_start;
  timeval tv;
  //drop off the fractional seconds...
  tv.tv_sec = diff.ticks()/time_duration::rep_type::res_adjust();
  //The following only works with microsecond resolution!
  tv.tv_usec = diff.fractional_seconds();
  return ACE_Time_Value(tv);
}

//-----------------------------------------------------------------------------
// Manipulate string
//-----------------------------------------------------------------------------
void datetime::toSemi(std::string& retstr)
{
  retstr = to_iso_string(m_tv.date()) +
           to_iso_string(m_tv.time_of_day());
  retstr += _TX("00");
}

std::string datetime::toSemi()
{
  std::string ts = to_iso_string(m_tv.date()) +
                   to_iso_string(m_tv.time_of_day());
  ts += _TX("00");
  return ts;
}

void datetime::toString(std::string& rs)
{
  rs = to_simple_string(m_tv);
}

std::string datetime::toString()
{
  return to_simple_string(m_tv);
}

//-----------------------------------------------------------------------------
// Convert string of SEMI standard format to boost::pisix_time
//-----------------------------------------------------------------------------
void datetime::set(const std::string& rhs)
{
  if (rhs.size() == 16)
  { // SEMI TIMEFORMAT
    std::string timestr = rhs.substr(0, 8) + _TX("T");
    timestr += rhs.substr(8);
    m_tv = from_iso_string(timestr);
  }
  else if (rhs.size() == 12)
  { // SEMI TIMEFORMAT
    // ptime t(second_clock::local_time());
    // std::string yearstr = t.to_iso_string();
    // std::string timestr = yearstr.substr(0, 2);
    std::string timestr = _TX("20");
    timestr += rhs.substr(0, 6);
    timestr += _TX("T");
    timestr += rhs.substr(6);
    m_tv = from_iso_string(timestr);
  }
  else
  {
    m_tv = time_from_string(rhs);
  }
}

//-----------------------------------------------------------------------------
// Create boost::posix_time (static)
//-----------------------------------------------------------------------------
ptime datetime::create_time(struct tm * current)
{
  date d(current->tm_year + 1900, current->tm_mon + 1,
         current->tm_mday);
  time_duration td(current->tm_hour, current->tm_min,
                   current->tm_sec);
  return ptime(d, td);
}

//-----------------------------------------------------------------------------
// Convert boost::posix_time to time struct (static)
//-----------------------------------------------------------------------------
void datetime::to_localtime(ptime pt, struct tm& retm, long& millisec)
{
  date d(pt.date());
  date::ymd_type ymd = d.year_month_day();
  time_duration  td = pt.time_of_day();
  retm.tm_year = ymd.year - 1900;
  retm.tm_mon = ymd.month - 1;
  retm.tm_mday = ymd.day;
  retm.tm_hour = td.hours();
  retm.tm_min = td.minutes();
  retm.tm_sec = td.seconds();
  millisec = td.fractional_seconds();
}


//-----------------------------------------------------------------------------
// Calculate lapse seconds in the day. (static)
//-----------------------------------------------------------------------------
unsigned int datetime::lapse(time_t curtime)
{
  struct tm * newtime;
  struct tm begintime;
  time_t basetime;

  newtime = localtime(&curtime);            // Convert to local time.

  begintime.tm_mday  = newtime->tm_mday;    // Convert to begin of the day.
  begintime.tm_mon   = newtime->tm_mon;
  begintime.tm_year  = newtime->tm_year;
  begintime.tm_wday  = newtime->tm_wday;
  begintime.tm_yday  = newtime->tm_yday;
  begintime.tm_isdst = newtime->tm_isdst;
  begintime.tm_hour  = 0;
  begintime.tm_sec   = 0;
  begintime.tm_min   = 0;

  basetime = mktime(&begintime);

  double diffsec = difftime(curtime, basetime);
  unsigned int lapse = (unsigned int)diffsec;

  return lapse;
}

//-----------------------------------------------------------------------------
// Make log file name by date. (static)
//-----------------------------------------------------------------------------
int datetime::makeDateName(time_t curtime, string& tvname)
{
  struct tm * newtime;
  BCHAR  buf[16];

  newtime = localtime(&curtime);            // Convert to local time.
#if 0
  // Use logging per date
  _stprintf(buf, _TX("%04d%02d%02d"), newtime->tm_year + 1900,
                                      newtime->tm_mon + 1,
                                      newtime->tm_mday);
#else
  _stprintf(buf, _TX("%d%02d%02d%02d%02d%02d"), newtime->tm_year + 1900,
                                                newtime->tm_mon + 1,
                                                newtime->tm_mday,
                                                newtime->tm_hour,
                                                newtime->tm_min,
                                                newtime->tm_sec);
#endif
  tvname = buf;
  return 0;
}

} }  // *namespace* - jyugem::bee