// $Id$

//////////////////////////////////////////////////////////////////////////////
// Copyright (c) Andreas Huber Doenni 2002-2004.
// Use, modification and distribution are subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//////////////////////////////////////////////////////////////////////////////

//=============================================================================
/**
 *  @file    CIDRWState.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#pragma warning( disable: 4244 ) // '__w64 int'
#endif

#include "CIDRWState.h"
#include "jyugem/cidrw/cidrw.h"

using namespace jyugem::bee;
namespace fsm = boost::fsm;
namespace mpl = boost::mpl;

//
namespace jyugem { namespace cidrw
{

//-----------------------------------------------------------------------------
// State Machine
//

CIDRWState::CIDRWState(CIDRW * cidrw) :
  task_base(cidrw->uniqueName(), cidrw), m_alarmq(0)
{
}

CIDRWState::task_base::event_dictionary  CIDRWState::_event_dictionary;

static cstr_id _initialize_ ( "Initialize", 1 );
static cstr_id _running_    ( "Running", 2 );
static cstr_id _alarmStatus_( "AlarmStatus", 4 );
static cstr_id _idle_       ( "Idle", 5 );
static cstr_id _maintenance_( "Maintenance", 6 );
static cstr_id _busy_       ( "Busy", 7 );
static cstr_id _alarms_     ( "Alarms", 9 );
static cstr_id _noAlarms_   ( "NoAlarms", 10 );
static cstr_id _initAlarmStatus_  ( "InitAlarmStatus", 8 );
static cstr_id _operationalStatus_( "OperationalStatus", 3 );


void CIDRWState::InitEventDictionary()
{
  if (! _event_dictionary.empty())
    return ;

  _event_dictionary.insert(std::make_pair("init",  new EvInitialized()));
  _event_dictionary.insert(std::make_pair("read",  new EvRead()));
  _event_dictionary.insert(std::make_pair("write", new EvWrite()));
  _event_dictionary.insert(std::make_pair("diag",  new EvDiagnostic()));
  _event_dictionary.insert(std::make_pair("alarm", new EvAlarmSet()));
  _event_dictionary.insert(std::make_pair("clear", new EvAlarmClear()));
  _event_dictionary.insert(std::make_pair("reset", new EvReset()));
  _event_dictionary.insert(std::make_pair("complete", new EvComplete()));
  _event_dictionary.insert(std::make_pair("operate",  new EvOperating()));
  _event_dictionary.insert(std::make_pair("maintenance", new EvMaintenance()));

  // Initialize static name of states
  Initialize::custom_static_type_ptr( &_initialize_ );
  Running:: custom_static_type_ptr( &_running_ );
  Idle::    custom_static_type_ptr( &_idle_ );
  Busy::    custom_static_type_ptr( &_busy_ );
  Alarms::  custom_static_type_ptr( &_alarms_ );
  NoAlarms::custom_static_type_ptr( &_noAlarms_ );
  AlarmStatus::      custom_static_type_ptr( &_alarmStatus_ );
  Maintenance::      custom_static_type_ptr( &_maintenance_ );
  InitAlarmStatus::  custom_static_type_ptr( &_initAlarmStatus_ );
  OperationalStatus::custom_static_type_ptr( &_operationalStatus_ );

}

const cstr_id * CIDRWState::get_current_name()
{
  return this->current_state_name<ViewedState>();
}

//
fsm::result Alarms::react( const EvAlarmClear & )
{
  outermost_context().m_alarmq--;
  if (outermost_context().m_alarmq > 0)
    return discard_event();
  else
  {
    CIDRW * cidrw = outermost_context().cast_task<CIDRW>();
    cidrw->m_alarmStatus = 0;
    return transit< NoAlarms >();
  }
}

fsm::result Alarms::react( const EvAlarmSet & )
{
  outermost_context().m_alarmq++;
  CIDRW * cidrw = outermost_context().cast_task<CIDRW>();
  cidrw->m_alarmStatus = 1;
  return discard_event();
}

//
fsm::result NoAlarms::react( const EvAlarmSet & )
{
  outermost_context().m_alarmq++;

  CIDRW * cidrw = outermost_context().cast_task<CIDRW>();
  cidrw->m_alarmStatus = 1;

  return transit< Alarms >();
}



//
//-----------------------------------------------------------------------------
// Header
//
CIDRWHeadState::CIDRWHeadState(CIDRWHead * head) :
  task_base(head->uniqueName(), head), m_operating(true)
{
}

CIDRWHeadState::task_base::event_dictionary  CIDRWHeadState::_event_dictionary;

static cstr_id _initHead_( "InitHead", 1 );
static cstr_id _headOperating_( "HeadOperating", 2 );
static cstr_id _headNotOperating_( "HeadNotOperating", 3 );
static cstr_id _headIdle_( "HeadIdle", 4 );
static cstr_id _headBusy_( "HeadBusy", 5 );


void CIDRWHeadState::InitEventDictionary()
{
  if (! _event_dictionary.empty())
    return ;

  _event_dictionary.insert(std::make_pair("read",  new EvRead()));
  _event_dictionary.insert(std::make_pair("write", new EvWrite()));
  _event_dictionary.insert(std::make_pair("diag",  new EvDiagnostic()));
  _event_dictionary.insert(std::make_pair("complete", new EvComplete()));
  _event_dictionary.insert(std::make_pair("diag",  new EvDiagnostic()));

  // Initialize static name of states
  InitHead::custom_static_type_ptr( &_initHead_ );
  HeadOperating::custom_static_type_ptr( &_headOperating_ );
  HeadNotOperating::custom_static_type_ptr( &_headNotOperating_ );
  HeadIdle::custom_static_type_ptr( &_headIdle_ );
  HeadBusy::custom_static_type_ptr( &_headBusy_ );

}


} }  // *namespace* jyugem::cidrw
