// $Id$

//////////////////////////////////////////////////////////////////////////////
// Copyright (c) Andreas Huber Doenni 2002-2004.
// Use, modification and distribution are subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//////////////////////////////////////////////////////////////////////////////

//=============================================================================
/**
 *  @file    ControlJobState.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif

#include "ControlJobState.h"
#include "jyugem/cjm/ControlJob.h"

namespace bee = jyugem::bee;
namespace fsm = boost::fsm;
namespace mpl = boost::mpl;

//
namespace jyugem { namespace cjm
{

//
//-----------------------------------------------------------------------------
// State Machine
//
ControlJobState::ControlJobState(ControlJob * cjm)
  : task_base(cjm->uniqueName(), cjm), m_StartMode(cjm->m_StartMethod)
{
}

ControlJobState::task_base::event_dictionary  ControlJobState::_event_dictionary;

static cstr_id _completed_( "Completed", 1 );
static cstr_id _queued_( "Queued", 2 );
static cstr_id _active_( "Active", 3 );
static cstr_id _initActive_( "InitActive", 4 );
static cstr_id _selected_( "Selected", 5 );
static cstr_id _waitingForStart_( "WaitingForStart", 6 );
static cstr_id _executing_( "Executing", 7 );
static cstr_id _pause_( "Pause", 8 );

void ControlJobState::InitEventDictionary()
{
  if (! _event_dictionary.empty())
    return ;

  _event_dictionary.insert(std::make_pair("abort",   new EvAbort()));
  _event_dictionary.insert(std::make_pair("begin",   new EvBegin()));
  _event_dictionary.insert(std::make_pair("cancel",  new EvCancel()));
  _event_dictionary.insert(std::make_pair("complete",new EvComplete()));
  _event_dictionary.insert(std::make_pair("deselect",new EvDeselect()));
  _event_dictionary.insert(std::make_pair("hoq",    new EvHOQ()));
  _event_dictionary.insert(std::make_pair("pause",  new EvPause()));
  _event_dictionary.insert(std::make_pair("resume", new EvResume()));
  _event_dictionary.insert(std::make_pair("start",  new EvStart()));
  _event_dictionary.insert(std::make_pair("stop",   new EvStop()));

  // Initialize static name of states
  Completed::custom_static_type_ptr( &_completed_ );
  Queued::custom_static_type_ptr   ( &_queued_ );
  Active::custom_static_type_ptr   ( &_active_ );
  InitActive::custom_static_type_ptr( &_initActive_ );
  Selected::custom_static_type_ptr ( &_selected_ );
  WaitingForStart::custom_static_type_ptr( &_waitingForStart_ );
  Executing::custom_static_type_ptr( &_executing_ );
  Pause::custom_static_type_ptr    ( &_pause_ );
}

//
// Select state
//
fsm::result Selected::react( const EvEvalCondition & )
{
  ControlJob * job = outermost_context().cast_task<ControlJob>();
  if (job != null && job->autoStart())
    return transit< Executing >();      // Auto
  else
    return transit< WaitingForStart >();
}



} }  // *namespace* jyugem::cjm

