// $Id$

//=============================================================================
/**
 *  @file    CMSClass.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif

#include "jyugem/bee.h"
#include "jyugem/cms/Carrier.h"
#include "jyugem/cms/FrontEndModule.h"
#include "jyugem/cms/LoadPort.h"
#include "jyugem/cms/LoadPortGroup.h"

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <boost/python/implicit.hpp>
#include <boost/python/class.hpp>
#include <boost/python/enum.hpp>
#include <boost/python/overloads.hpp>
#include <boost/python/return_internal_reference.hpp>
#include <boost/shared_ptr.hpp>


using namespace boost::python;
//
namespace jyugem { namespace cms
{

struct cms_frontend_wrapper
{
  static void wrap()
  {
    objects::class_value_wrapper<
      boost::shared_ptr<LoadPort>
        , objects::make_ptr_instance<LoadPort,
          objects::pointer_holder<boost::shared_ptr<LoadPort>, LoadPort> >
      >();
      ;

    class_< FrontEndModule >("FrontEndModule")
      .def("open",          &FrontEndModule::open)
      .def("loadport",      &FrontEndModule::loadport)
      ;
  }
};

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(lp_current_state_overloads, LoadPort::current_state, 0, 1)

struct cms_loadport_wrapper
{
  static void wrap()
  {
    objects::class_value_wrapper<
      boost::shared_ptr<Carrier>
        , objects::make_ptr_instance<Carrier,
          objects::pointer_holder<boost::shared_ptr<Carrier>, Carrier> >
      >();
      ;


    scope outer
      = class_< LoadPort >("LoadPort")
        .def_readwrite("BypassReadID",      &LoadPort::m_bypassReadID)
        .def_readwrite("PortTransferState", &LoadPort::m_portTransferState)
        .def_readwrite("AccessMode",        &LoadPort::m_accessMode)
        .def_readwrite("ManualTimer",       &LoadPort::m_manualTimer)
        .def_readwrite("ManualInType",      &LoadPort::m_manualInType)
        .def_readwrite("ManualOutType",     &LoadPort::m_manualOutType)
        .def_readwrite("UndockStyle",       &LoadPort::m_undockStyle)
        .def_readwrite("UnclampStyle",      &LoadPort::m_unclampStyle)
        .def_readwrite("Carrier",           &LoadPort::m_carrier)
        .def_readwrite("ReservationState",  &LoadPort::m_reservationState)
        .def_readwrite("PortAssosiationState", &LoadPort::m_portAssosiationState)
        .def_readwrite("PortID",            &LoadPort::m_portID)
        .def_readwrite("Reason",            &LoadPort::m_reason)
        .def("open",          &LoadPort::open0)
        .def("update",        &LoadPort::update)
        .def("current_state", &LoadPort::current_state, lp_current_state_overloads())
        .def("carrier",       &LoadPort::carrier)
        .def("onCarrier",     &LoadPort::onCarrier)
        .def("offCarrier",    &LoadPort::offCarrier)
      ;

    enum_<LoadPort::Orthogonal>("region")
        .value("TRANSFER",    LoadPort::Orthogonal::TRANSFER)
        .value("ACCESS",      LoadPort::Orthogonal::ACCESS)
        .value("RESERVATION", LoadPort::Orthogonal::RESERVATION)
        .value("ASSOCIATION", LoadPort::Orthogonal::ASSOCIATION)
        .export_values()
        ;

  }
};

struct cms_loadport_group_wrapper
{
  static void wrap()
  {
    class_< LoadPortGroup >("LoadPortGroup", init<const std::string&>())
      .def("name",   &LoadPortGroup::name)
      .def("ports",  &LoadPortGroup::ports)
      ;

  }
};

//
// Carrier class
//
struct cms_carrier_content_wrapper
{
  static void wrap()
  {
    class_< Content >("Content" )
      .add_property("LotID", &Content::LotID)
      .add_property("SubstrateID", &Content::SubstrateID)
      ;

    // Compile check only...
    class_<std::vector<Content *> >("ContentVec")
      .def(vector_indexing_suite<std::vector<Content *> >())
      ;
  }
};


struct cms_carrier_wrapper
{
  static void wrap()
  {
    // Compile check only...
    class_<std::vector<int> >("SlotVec")
      .def(vector_indexing_suite<std::vector<int> >())
      ;

    scope outer
      = class_< Carrier >("Carrier")
        .def_readwrite("Capacity",       &Carrier::m_Capacity)
        .def_readwrite("SubstrateCount", &Carrier::m_SubstrateCount)
        .def_readwrite("CarrierIDStatus",&Carrier::m_CarrierIDStatus)
        .def_readwrite("CarrierAccessingStatus", &Carrier::m_CarrierAccessingStatus)
        .def_readwrite("ContentMap",    &Carrier::m_ContentMap)
        .def_readwrite("LocationID",    &Carrier::m_LocationID)
        .def_readwrite("SlotMap",       &Carrier::m_SlotMap)
        .def_readwrite("SlotMapStatus", &Carrier::m_SlotMapStatus)
        .def_readwrite("Usage",         &Carrier::m_Usage)
        .def("open",          &Carrier::open0)
        .def("update",        &Carrier::update)
        .def("current_state", &Carrier::current_state)
      ;

    enum_<Carrier::Orthogonal>("region")
        .value("IDSTATUS",    Carrier::Orthogonal::IDSTATUS)
        .value("SLOTMAP",     Carrier::Orthogonal::SLOTMAP)
        .value("ACCESS",      Carrier::Orthogonal::ACCESS)
        .export_values()
        ;

  }
};


//
void init_module()
{
  cms_frontend_wrapper::wrap();
  cms_loadport_group_wrapper::wrap();
  cms_loadport_wrapper::wrap();
  cms_carrier_content_wrapper::wrap();
  cms_carrier_wrapper::wrap();
}

} } // *namespace* jyugem::cms

BOOST_PYTHON_MODULE(cms)
{

  jyugem::cms::init_module();

}